@@ -379,14 +379,13 @@ def read_out_cs_ind(fname='out.cs.individuals'):
 def write_db_optzer(db,fname='db.optzer.json'):
     """Write db in JSON format."""
     with open(fname,'w') as f:
-        f.write(db.to_json())
+        f.write(db.to_json(orient='records',lines=True))
     return None
 
 def read_db_optzer(fname='db.optzer.json'):
     """Read db_optzer in JSON format."""
     try:
-        db = pd.read_json(fname)
+        db = pd.read_json(fname, orient='records', lines=True)
     except:
         raise
     return db
-
