@@ -13,13 +13,15 @@ from docopt import docopt
 import numpy as np
 import random
 import copy
-from multiprocessing import Process, Pool
+#from multiprocessing import Process, Pool
+from multiprocess import Pool
 from time import time
 import pandas as pd
 
-from optzer.individual import Individual, ind_from_db
+from optzer.individual import Individual, ind_from_db, update_slims
 from optzer.io import write_db_optzer, read_db_optzer
 
+
 __author__ = "RYO KOBAYASHI"
 __version__ = "221228"
 
@@ -157,7 +159,7 @@ class TPE:
         #...Try to restart by loading db.optzer.json if exists
         if os.path.exists(_fname_db):
             try:
-                self.history_db = pd.read_json(_fname_db)
+                self.history_db = read_db_optzer(_fname_db)
             except Exception as e:
                 print(e)
                 print(f'\n !!! Failed to load {_fname_db} for restart. !!!'
@@ -240,7 +242,9 @@ class TPE:
         #...Dump the DB once it is updated.
         write_db_optzer(self.history_db,fname=_fname_db)
 
-        # self.slims = update_slims(self.slims, self.hlims, self.history_db)
+        self.slims = update_slims(self.slims, self.hlims,
+                                  self.history_db,
+                                  ntops=max(100,int(len(self.history_db)*self.gamma)))
         
         #...Check best
         self.keep_best()
@@ -291,7 +295,9 @@ class TPE:
             #...Dump the DB once it is updated.
             write_db_optzer(self.history_db,fname=_fname_db)
 
-            # self.slims = update_slims(self.slims, self.hlims, self.history_db)
+            self.slims = update_slims(self.slims, self.hlims,
+                                      self.history_db,
+                                      ntops=max(100,int(len(self.history_db)*self.gamma)))
 
             #...Check best
             best_updated = False
@@ -313,7 +319,7 @@ class TPE:
 
     def _write_step_info(self,istp,starttime):
         print(' step,time,best_iid,best_loss,vars='
-              +' {0:6d} {1:8.1f} {2:5d} {3:8.4f}'.format(istp+1,
+              +' {0:6d} {1:8.1f} {2:5d} {3:8.4f}'.format(istp,
                                                          time()-starttime,
                                                          self.bestsmpl.iid,
                                                          self.bestsmpl.loss),end="")
@@ -353,8 +359,10 @@ class TPE:
         ntrial = max(self.ntrial,self.nbatch)
         for idim in range(self.ndim):
             key = self.vnames[idim]
-            xhmin = self.hlims[key][0]
-            xhmax = self.hlims[key][1]
+            # xhmin = self.hlims[key][0]
+            # xhmax = self.hlims[key][1]
+            xhmin = self.slims[key][0]
+            xhmax = self.slims[key][1]
             xlowsrt = np.sort(Xlow[:,idim])
             npnt = len(xlowsrt)
             #...Determine smoothness parameter, h, by Silverman's method
@@ -431,8 +439,10 @@ class TPE:
         xcandidates = np.empty((self.nbatch,self.ndim))
         for idim in range(self.ndim):
             key = self.vnames[idim]
-            xhmin = self.hlims[key][0]
-            xhmax = self.hlims[key][1]
+            # xhmin = self.hlims[key][0]
+            # xhmax = self.hlims[key][1]
+            xhmin = self.slims[key][0]
+            xhmax = self.slims[key][1]
             xsrt = np.sort(xtmps[:,idim])
             #...Determine smoothness parameter, h, by Silverman's method
             q75, q25 = np.percentile(xsrt, [75,25])
